.hll { background-color: #ffffcc; }
.err { background-color: #ffcccc; } /* Error */

.c   { color: #1293de; } /* Comment */
.cm  { color: #1293de; } /* Comment.Multiline */
.cp  { color: #9f6fff; } /* Comment.Preproc */
.c1  { color: #1293de; } /* Comment.Single */
.cs  { color: #1293de; } /* Comment.Special */
.gd  { color: #111111; } /* Generic.Deleted */
.gr  { color: #ff405a; } /* Generic.Error */
.gh  { color: #111111; } /* Generic.Heading */
.gi  { color: #111111; } /* Generic.Inserted */
.go  { color: #111111; } /* Generic.Output */
.gp  { color: #ff405a; } /* Generic.Prompt */
.gt  { color: #ff405a; } /* Generic.Traceback */
.gu  { color: #111111; } /* Generic.Subheading */
.k   { color: #9f6fff; } /* Keyword */
.kc  { color: #00ba88; } /* Keyword.Constant */
.kd  { color: #9f6fff; } /* Keyword.Declaration */
.kn  { color: #9f6fff; } /* Keyword.Namespace */
.kp  { color: #9f6fff; } /* Keyword.Pseudo */
.kr  { color: #9f6fff; } /* Keyword.Reserved */
.kt  { color: #00bb29; } /* Keyword.Type */
.m   { color: #e072cd; } /* Literal.Number */
.mf  { color: #e072cd; } /* Literal.Number.Float */
.mh  { color: #e072cd; } /* Literal.Number.Hex */
.mi  { color: #e072cd; } /* Literal.Number.Integer */
.il  { color: #e072cd; } /* Literal.Number.Integer.Long */
.mo  { color: #e072cd; } /* Literal.Number.Oct */
.na  { color: #c4a000; } /* Name.Attribute */
.nb  { color: #ff6f87; } /* Name.Builtin */
.bp  { color: #c4a000; } /* Name.Builtin.Pseudo */
.nc  { color: #1d77ff; } /* Name.Class */
.no  { color: #00ba88; } /* Name.Constant */
.nd  { color: #00bb29; } /* Name.Decorator */
.ni  { color: #111111; } /* Name.Entity */
.ne  { color: #00bb29; } /* Name.Exception */
.nf  { color: #1d77ff; } /* Name.Function */
.nl  { color: #111111; } /* Name.Label */
.nn  { color: #111111; } /* Name.Namespace */
.nt  { color: #111111; } /* Name.Tag */
.nv  { color: #c4a000; } /* Name.Variable */
.vc  { color: #c4a000; } /* Name.Variable.Class */
.vg  { color: #c4a000; } /* Name.Variable.Global */
.vi  { color: #c4a000; } /* Name.Variable.Instance */
.o   { color: #111111; } /* Operator */
.ow  { color: #9f6fff; } /* Operator.Word */
.s   { color: #e072cd; } /* Literal.String */
.sb  { color: #e072cd; } /* Literal.String.Backtick */
.sc  { color: #e072cd; } /* Literal.String.Char */
.sd  { color: #e072cd; } /* Literal.String.Doc */
.s2  { color: #e072cd; } /* Literal.String.Double */
.se  { color: #e072cd; } /* Literal.String.Escape */
.sh  { color: #e072cd; } /* Literal.String.Heredoc */
.si  { color: #e072cd; } /* Literal.String.Interpol */
.sx  { color: #e072cd; } /* Literal.String.Other */
.sr  { color: #e072cd; } /* Literal.String.Regex */
.s1  { color: #e072cd; } /* Literal.String.Single */
.ss  { color: #e072cd; } /* Literal.String.Symbol */
.w   { color: #111111; } /* Text.Whitespace */

.ge  { font-style: italic; } /* Generic.Emph */
.gs  { font-weight: bold; } /* Generic.Strong */
.nc  { text-decoration: underline; } /* Name.Class */
.nf  { text-decoration: underline; } /* Name.Function */
